@@ -31,14 +31,20 @@ data PubKeyALG =
     | PubKeyALG_RSAPSS      -- ^ RSA PSS Key algorithm (RFC 3447)
     | PubKeyALG_DSA         -- ^ DSA Public Key algorithm
     | PubKeyALG_EC          -- ^ ECDSA & ECDH Public Key algorithm
+    | PubKeyALG_X25519      -- ^ ECDH 25519 key agreement
+    | PubKeyALG_X448        -- ^ ECDH 448 key agreement
+    | PubKeyALG_Ed25519     -- ^ EdDSA 25519 signature algorithm
+    | PubKeyALG_Ed448       -- ^ EdDSA 448 signature algorithm
     | PubKeyALG_DH          -- ^ Diffie Hellman Public Key algorithm
     | PubKeyALG_Unknown OID -- ^ Unknown Public Key algorithm
     deriving (Show,Eq)
 
--- | Signature Algorithm often composed of
--- a public key algorithm and a hash algorithm
+-- | Signature Algorithm, often composed of a public key algorithm and a hash
+-- algorithm.  For some signature algorithms the hash algorithm is intrinsic to
+-- the public key algorithm and is not needed in the data type.
 data SignatureALG =
       SignatureALG HashALG PubKeyALG
+    | SignatureALG_IntrinsicHash PubKeyALG
     | SignatureALG_Unknown OID
     deriving (Show,Eq)
 
@@ -47,6 +53,10 @@ instance OIDable PubKeyALG where
     getObjectID PubKeyALG_RSAPSS = [1,2,840,113549,1,1,10]
     getObjectID PubKeyALG_DSA    = [1,2,840,10040,4,1]
     getObjectID PubKeyALG_EC     = [1,2,840,10045,2,1]
+    getObjectID PubKeyALG_X25519    = [1,3,101,110]
+    getObjectID PubKeyALG_X448      = [1,3,101,111]
+    getObjectID PubKeyALG_Ed25519   = [1,3,101,112]
+    getObjectID PubKeyALG_Ed448     = [1,3,101,113]
     getObjectID PubKeyALG_DH     = [1,2,840,10046,2,1]
     getObjectID (PubKeyALG_Unknown oid) = oid
 
@@ -71,6 +81,8 @@ sig_table =
         , ([2,16,840,1,101,3,4,2,4],  SignatureALG HashSHA224 PubKeyALG_RSAPSS)
         , ([2,16,840,1,101,3,4,3,1],  SignatureALG HashSHA224 PubKeyALG_DSA)
         , ([2,16,840,1,101,3,4,3,2],  SignatureALG HashSHA256 PubKeyALG_DSA)
+        , ([1,3,101,112], SignatureALG_IntrinsicHash PubKeyALG_Ed25519)
+        , ([1,3,101,113], SignatureALG_IntrinsicHash PubKeyALG_Ed448)
         ]
 
 oidSig :: OID -> SignatureALG
@@ -99,6 +111,6 @@ instance ASN1Object SignatureALG where
         Right (oidSig hash1, xs)
     fromASN1 _ =
         Left "fromASN1: X509.SignatureALG: unknown format"
-    toASN1 signatureAlg@(SignatureALG hashAlg PubKeyALG_RSAPSS) = \xs -> Start Sequence:OID [1,2,840,113549,1,1,10]:Start Sequence:Start (Container Context 0):Start Sequence:OID (sigOID signatureAlg):End Sequence:End (Container Context 0):Start (Container Context 1): Start Sequence:OID [1,2,840,113549,1,1,8]:Start Sequence:OID (sigOID signatureAlg):End Sequence:End Sequence:End (Container Context 1):Start (Container Context 2):IntVal (saltLen hashAlg):End (Container Context 2):End Sequence:End Sequence:xs
-    toASN1 signatureAlg@(SignatureALG _ _) = \xs -> Start Sequence:OID (sigOID signatureAlg):Null:End Sequence:xs
     toASN1 (SignatureALG_Unknown oid) = \xs -> Start Sequence:OID oid:Null:End Sequence:xs
+    toASN1 signatureAlg@(SignatureALG hashAlg PubKeyALG_RSAPSS) = \xs -> Start Sequence:OID [1,2,840,113549,1,1,10]:Start Sequence:Start (Container Context 0):Start Sequence:OID (sigOID signatureAlg):End Sequence:End (Container Context 0):Start (Container Context 1): Start Sequence:OID [1,2,840,113549,1,1,8]:Start Sequence:OID (sigOID signatureAlg):End Sequence:End Sequence:End (Container Context 1):Start (Container Context 2):IntVal (saltLen hashAlg):End (Container Context 2):End Sequence:End Sequence:xs
+    toASN1 signatureAlg = \xs -> Start Sequence:OID (sigOID signatureAlg):Null:End Sequence:xs
