@@ -18,7 +18,7 @@ module Data.X509.CRL
 
 import Control.Applicative
 
-import Data.Hourglass (DateTime)
+import Data.Hourglass (DateTime, TimezoneOffset(..))
 import Data.ASN1.Types
 
 import Data.X509.DistinguishedName
@@ -54,7 +54,7 @@ instance ASN1Object RevokedCertificate where
         Right (RevokedCertificate serial t (Extensions Nothing), xs)
     fromASN1 l = Left ("fromASN1: X509.RevokedCertificate: unknown format:" ++ show l)
     toASN1 (RevokedCertificate serial time _) = \xs ->
-        Start Sequence : IntVal serial : ASN1Time TimeGeneralized time Nothing : End Sequence : xs
+        Start Sequence : IntVal serial : ASN1Time TimeGeneralized time (Just (TimezoneOffset 0)) : End Sequence : xs
 
 parseCRL :: ParseASN1 CRL
 parseCRL = do
@@ -66,10 +66,10 @@ parseCRL = do
         <*> getRevokedCertificates
         <*> getObject
   where getVersion (IntVal v) = return $ fromIntegral v
-        getVersion _          = throwError "unexpected type for version"
+        getVersion _          = throwParseError "unexpected type for version"
 
         getThisUpdate (ASN1Time _ t1 _) = return t1
-        getThisUpdate _                 = throwError "bad this update format, expecting time"
+        getThisUpdate _                 = throwParseError "bad this update format, expecting time"
 
         getNextUpdate = getNextMaybe timeOrNothing
 
@@ -83,8 +83,8 @@ encodeCRL crl xs =
     [IntVal $ crlVersion crl] ++
     toASN1 (crlSignatureAlg crl) [] ++
     toASN1 (crlIssuer crl) [] ++
-    [ASN1Time TimeGeneralized (crlThisUpdate crl) Nothing] ++
-    (maybe [] (\t -> [ASN1Time TimeGeneralized t Nothing]) (crlNextUpdate crl)) ++
+    [ASN1Time TimeGeneralized (crlThisUpdate crl) (Just (TimezoneOffset 0))] ++
+    (maybe [] (\t -> [ASN1Time TimeGeneralized t (Just (TimezoneOffset 0))]) (crlNextUpdate crl)) ++
     [Start Sequence] ++
     revoked ++
     [End Sequence] ++
